def construct_range(worksheet_name, start, end=None):
    range_ = "'" + worksheet_name + "'" + "!" + start
    if end:
        range_ = range_ + ":" + end
    return range_


def column_string(n):
    string = ""
    while n > 0:
        n, remainder = divmod(n - 1, 26)
        string = chr(65 + remainder) + string
    return string


def delete_none(rows):
    for i in range(len(rows)):
        for j in range(len(rows[i])):
            if rows[i][j] is None:
                rows[i][j] = ""
    return rows



from pyspreadsheet.core.manage_sheet import get_row_count
from .tool import construct_range, delete_none


def write(spreadsheet, sheet_id, data):
    worksheet_name = data["worksheet_name"]
    columns_name = data["columns_name"]
    rows = data["rows"]
    rows = delete_none(rows)
    values = [columns_name] + rows

    body = {
        "value_input_option": "USER_ENTERED",
        "data": [

            {
                "range": construct_range(worksheet_name, 'A1'),
                "majorDimension": "ROWS",
                "values": values,
            }
        ]
    }
    response = spreadsheet.account.values().batchUpdate(spreadsheetId=sheet_id,
                                            body=body).execute()
    row_count = get_row_count(spreadsheet, sheet_id, worksheet_name)
    if row_count > len(values):
        clean_row = ["" for r in columns_name]
        clean_rows = [clean_row for i in range(row_count - len(values))]
        clean_body = {
            "value_input_option": "USER_ENTERED",
            "data": [
                {
                    "range": construct_range(worksheet_name, 'A' + str(len(values) + 1)),
                    "majorDimension": "ROWS",
                    "values": clean_rows,
                }
            ]
        }
        spreadsheet.account.values().batchUpdate(spreadsheetId=sheet_id,
                                     body=clean_body).execute()

    return response



import datetime

import decimal
import os
import copy
import json
import requests
import googleapiclient
from googleauthentication import GoogleAuthentication
from dbstream import DBStream

from pyspreadsheet.core import write
from pyspreadsheet.core.manage_sheet import add_worksheet

from pyspreadsheet.core.read import check_availability_column


class Spreadsheet:
    def __init__(self, googleauthentication: GoogleAuthentication, dbstream: DBStream = None):
        self.googleauthentication = googleauthentication
        self.dbstream = dbstream
        self.account = googleauthentication.get_account("sheets").spreadsheets()

    def send(self, sheet_id, data):
        try:
            available_response = check_availability_column(self, sheet_id, data)
        except googleapiclient.errors.HttpError:
            add_worksheet(self, sheet_id, data["worksheet_name"])
            available_response = {
                "completely_available": True
            }
        available = available_response["completely_available"]
        if not available:
            user_response = input("Columns " +
                                  ", ".join(available_response["warning_columns"]) +
                                  " will be overwritten, do you want to continue ?" +
                                  "\nType yes to continue\n")
            if user_response.lower() not in ["yes", "y"]:
                return 0
        write.write(self, sheet_id, data)
        data_copy = copy.deepcopy(data)
        url = os.environ.get("MONITORING_EXPORT_URL")
        if url:
            body = {
                "dbstream_instance_id": self.googleauthentication.dbstream.dbstream_instance_id,
                "instance_name": self.googleauthentication.dbstream.instance_name,
                "client_id": self.googleauthentication.dbstream.client_id,
                "instance_type_prefix": self.googleauthentication.dbstream.instance_type_prefix,
                "sheet_id": sheet_id,
                "worksheet_name": data_copy["worksheet_name"],
                "nb_rows": len(data_copy["rows"]),
                "nb_columns": len(data_copy["columns_name"]),
                "timestamp": str(datetime.datetime.now()),
                "ssh_tunnel": True if self.googleauthentication.dbstream.ssh_tunnel else False,
                "local_absolute_path": os.getcwd()
            }
            r = requests.post(url=url, data=json.dumps(body))
            print(r.status_code)
        return 0

    def query_to_sheet(self, sheet_id, worksheet_name, query):
        if not self.dbstream:
            print("No DBStream set")
            exit()
        items = self.dbstream.execute_query(query)
        if not items:
            return 0
        columns_name = list(items[0].keys())
        rows = [list(i.values()) for i in items]
        for row in rows:
            for i in range(len(row)):
                r = row[i]
                if isinstance(r, datetime.datetime):
                    row[i] = str(r)
                if isinstance(r, decimal.Decimal):
                    row[i] = float(r)
        data = {
            "worksheet_name": worksheet_name,
            "columns_name": columns_name,
            "rows": rows
        }
        self.send(sheet_id, data)
        return 0
